/* eslint no-console: off */

const Base = require('./lib/base-page');

/**
 * ==========================================
 *  WAIT
 * ==========================================
 */

/**
 * Sleep/pause page (this.api)
 * @param {String} page
 * @param {Integer} timeSleep
 */
function pauseSleep(page, timeSleep) {
  return page.pause(timeSleep);
}

/**
 * Wait until element is present
 * @param {String} page
 * @param {String} elementSelector
 */
function waitElementPresent(page, elementSelector) {
  const wait = new Base.Wait(page, elementSelector);
  wait.waitElementPresent();
}

/**
 * @param {String} page
 * @param {String} elementSelector
 */
function waitElementVisible(page, elementSelector) {
  const wait = new Base.Wait(page, elementSelector);
  wait.waitElementVisible();
}

/**
 * wait element enabled until timeout
 * @param {*} page
 * @param {*} elementSelector
 */
function waitUntilElementEnabled(page, elementSelector) {
  const wait = new Base.Wait(page, elementSelector);
  wait.waitElementEnable();
}

/**
 * wait element disabled until timeout
 * @param {*} page
 * @param {*} elementSelector
 */
function waitUntilElementDisabled(page, elementSelector) {
  const wait = new Base.Wait(page, elementSelector);
  wait.waitElementDisable();
}

/**
 * ==========================================
 * ACTION
 * ==========================================
 */

/**
 * set and access set_url
 * @param {String} baseURL
 */
function setURL(baseURL) {
  console.log('========================================');
  console.log(`|| Accessing ${baseURL} ||`);
  console.log('========================================');
  return baseURL;
}

/**
 * Maximize window size
 * @param {String} page
 */
function maxWindow(page) {
  return page.resizeWindow(1920, 1080);
}

/**
 * Switch browser tab based on tab number
 * @param {String} page
 * @param {String} tabNumber
 */
function switchTabWindow(page, tabNumber) {
  const BrowserCommon = new Base.BrowserCommon(page, tabNumber);
  BrowserCommon.switchTabWindow();
}

/**
 * Go to specific element
 * @param {String} page
 * @param {String} elementSelector
 */
function scrollToElement(page, elementSelector) {
  const elementCommon = new Base.ElementCommon(page, elementSelector);
  waitElementVisible(page, elementSelector);
  elementCommon.scrollToElement();
}

/**
 * Click a button
 * @param {String} page
 * @param {String} elementSelector
 */
function clickButton(page, elementSelector) {
  const button = new Base.Button(page, elementSelector);
  waitElementVisible(page, elementSelector);
  button.clickButton();
}

/**
 * Click a button if it exists
 * If it does not exist continue the test
 * @param {String} page
 * @param {String} elementSelector
 */
function clickButtonOptional(page, client, elementSelector) {
  const buttonOptional = new Base.ButtonOptional(page, client, elementSelector);
  buttonOptional.clickButtonOptional();
}

/**
 * Click a button without waiting
 * @param {String} page
 * @param {String} elementSelector
 */
function clickButtonWithoutWait(page, elementSelector) {
  const button = new Base.Button(page, elementSelector);
  button.clickButton();
}

/**
 * Click a label
 * @param {String} page
 * @param {String} elementSelector
 */
function clickLabel(page, elementSelector) {
  const label = new Base.Label(page, elementSelector);
  waitElementVisible(page, elementSelector);
  label.clickLabel();
}

/**
 * Click a label if it exists
 * If it does not exist continue the test
 * @param {String} page
 * @param {String} elementSelector
 */
function clickLabelOptional(page, client, elementSelector) {
  const labelOptional = new Base.LabelOptional(page, client, elementSelector);
  labelOptional.clickLabelOptional();
}

/**
 * Click a link text
 * @param {String} page
 * @param {String} elementSelector
 */
function clickLinkText(page, elementSelector) {
  const linktText = new Base.LinkText(page, elementSelector);
  waitElementVisible(page, elementSelector);
  linktText.clickLinkText();
}

/**
 * Click a link by text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} textElement
 */
function clickLinkByText(page, elementSelector, textElement) {
  const linktText = new Base.LinkText(page, elementSelector, textElement);
  waitElementVisible(page, elementSelector);
  linktText.clickLinkByText();
}

/**
 * Click a link text without waiting
 * @param {String} page
 * @param {String} elementSelector
 */
function clickLinkTextWithoutWait(page, elementSelector) {
  const linktText = new Base.Button(page, elementSelector);
  waitElementVisible(page, elementSelector);
  linktText.clickLinkText();
}

/**
 * Click a division tag
 * @param {String} page
 * @param {String} elementSelector
 */
function clickDivElement(page, elementSelector) {
  const div = new Base.Div(page, elementSelector);
  waitElementVisible(page, elementSelector);
  div.clickDivElement();
}

/**
 * Click a span tag
 * @param {String} page
 * @param {String} elementSelector
 */
function clickSpanElement(page, elementSelector) {
  const span = new Base.Span(page, elementSelector);
  waitElementVisible(page, elementSelector);
  span.clickSpanElement();
}

/**
 * Click a icon tag
 * @param {String} page
 * @param {String} elementSelector
 */
function clickIconElement(page, elementSelector) {
  const icon = new Base.Icon(page, elementSelector);
  waitElementVisible(page, elementSelector);
  icon.clickIconElement();
}

/**
 * Click a list tag
 * @param {String} page
 * @param {String} elementSelector
 */
function clickListElement(page, elementSelector) {
  const list = new Base.List(page, elementSelector);
  waitElementVisible(page, elementSelector);
  list.clickListElement();
}

/**
 * Click an image tag
 * @param {String} page
 * @param {String} elementSelector
 */
function clickImageElement(page, elementSelector) {
  const list = new Base.Image(page, elementSelector);
  waitElementVisible(page, elementSelector);
  list.clickImageElement();
}

/**
 * Set value to text field
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} value
 */
function setValueTextField(page, elementSelector, value) {
  const textField = new Base.TextField(page, elementSelector, value);
  waitElementVisible(page, elementSelector);
  textField.setValueTextField();
}

/**
 * Set value to text field then press Enter key
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} value
 * @param {String} browser
 */
function setValueTextFieldThenEnter(page, elementSelector, value, browser) {
  page.clearValue(elementSelector, () => {
    page.click(elementSelector, () => {
      page.setValue(elementSelector, [value, browser.Keys.ENTER]);
    });
  });
}

/**
 * Set value to text area
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} value
 */
function setValueTextArea(page, elementSelector, value) {
  const textArea = new Base.TextArea(page, elementSelector, value);
  waitElementVisible(page, elementSelector);
  textArea.setValueTextArea();
}

/**
 * choose an option from dropdown list
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} selectedOption
 */
function chooseOptionValue(page, elementSelector, selectedOption) {
  const dropdown = new Base.Dropdown(page, elementSelector, selectedOption);
  waitElementVisible(page, elementSelector);
  dropdown.chooseOptionValue(page, elementSelector, selectedOption);
}

/**
 * choose an option from dropdown list using css class selector
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} selectedOption
 */
function chooseOptionClass(page, elementSelector, selectedOption) {
  const dropdown = new Base.Dropdown(page, elementSelector, selectedOption);
  waitElementVisible(page, elementSelector);
  dropdown.chooseOptionClass(page, elementSelector, selectedOption);
}

/**
 * Select radio button
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} value
 */
function selectRadioButton(page, elementSelector) {
  const radioButton = new Base.RadioButton(page, elementSelector);
  radioButton.selectRadioButton();
}

/**
 * Select radio button by Value
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} value
 */
function selectRadioButtonByValue(page, elementSelector, value) {
  const radioButton = new Base.RadioButton(page, elementSelector, value);
  radioButton.selectRadioButtonByValue();
}

/**
 * Select checkbox
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} value
 */
function selectCheckbox(page, elementSelector) {
  const radioButton = new Base.CheckBox(page, elementSelector);
  radioButton.selectCheckbox();
}

/**
 * Select checkbox by Value
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} value
 */
function selectCheckboxByValue(page, elementSelector, value) {
  const radioButton = new Base.CheckBox(page, elementSelector, value);
  radioButton.selectCheckboxByValue();
}

/**
 * Upload file
 * example: elementSelector = .addImageButtonLarge>input[type="file"]
 * pathFile = '/media/upload/1.jpg'
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} pathFile
 */
function uploadFile(page, elementSelector, pathFile) {
  const fileOps = new Base.FileOps(page, elementSelector, pathFile);
  waitElementPresent(page, elementSelector);
  fileOps.uploadFile();
}

/**
 * Set value to text field using keyboard press
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} words
 */
function keyboardPress(page, elementSelector, words) {
  const keyboard = new Base.Keyboard(page, elementSelector, words);
  keyboard.keyboardPressOnTextField();
}

/**
 * Set value to text field using keyboard press with delayed time 500 ms
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} words
 */
function keyboardPressWithDelay(page, elementSelector, words) {
  const keyboard = new Base.Keyboard(page, elementSelector, words);
  keyboard.keyboardPresswithDelayOnTextField();
}

/**
 * Set value to other field type using keyboard press
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} words
 */
function keyboardPressOnOthers(page, elementSelector, words) {
  const keyboard = new Base.Keyboard(page, elementSelector, words);
  keyboard.keyboardPressOnOthers();
}

/**
 * Set value to other field type using keyboard press
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} words
 */
function keyboardPressWithDelayOnOthers(page, elementSelector, words) {
  const keyboard = new Base.Keyboard(page, elementSelector, words);
  keyboard.keyboardPresswithDelayOnOthers();
}

/**
 * ==========================================
 * ASSERT
 * ==========================================
 */

/**
 * Assert URL equals to expected text
 * @param {String} page
 * @param {String} expectedText
 */
function assertURLEquals(page, expectedText) {
  const url = new Base.URL(page, expectedText);
  url.assertURLEquals();
}

/**
 * Assert URl contains expected text
 * @param {String} page
 * @param {String} expectedText
 */
function assertURLContains(page, expectedText) {
  const url = new Base.URL(page, expectedText);
  url.assertURLContains();
}

/**
 * Assert page title
 * @param {String} page
 * @param {String} expectedText
 */
function assertPageTitle(page, expectedText) {
  const pageComon = new Base.PageCommon(page, expectedText);
  return pageComon.assertPageTitle(page, expectedText);
}

/**
 * Check if given element contains the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function assertContainsText(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.assertContainsText();
}

/**
 * Check if given label contains the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function assertLabelContainsText(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.assertLabelContainsText();
}

/**
 * Check if given paragraph contains the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function assertParagraphContainsText(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.assertParagraphContainsText();
}

/**
 * Check if given span contains the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function assertSpanContainsText(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.assertSpanContainsText();
}

function assertLinkTextContainsText(page, elementSelector, expectedText) {
  const linktText = new Base.LinkText(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return linktText.assertLinkTextContainsText();
}

/**
 * Assert if given element is visible
 * @param {String} page
 * @param {String} elementSelector
 */
function assertVisible(page, elementSelector) {
  const assert = new Base.AssertElement(page, elementSelector);
  return assert.assertElementVisible();
}
/**
 * Assert if a list/collection of elements on screen contains certain text
 * Accept both xpath and css element selector
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function assertListOfElementsContainsText(page, client, elementSelector, expectedText) {
  const assert = new Base.AssertElement(page, elementSelector);
  return assert.assertElementsContainText(client, expectedText);
}

/**
 * Search text from given multiple label elements
 * @param {String} client
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function searchTextByElements(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.searchTextByLabelElements();
}

/**
 * ==========================================
 * EXPECT
 * ==========================================
 */

/**
 * Expect if element is visible
 * @param {String} page
 * @param {String} elementSelector
 */
function expectVisible(page, elementSelector) {
  const expectElement = new Base.ExpectElement(page, elementSelector);
  waitElementVisible(page, elementSelector);
  return expectElement.expectElementVisible();
}

/**
 * Expect if element is not be visible
 * @param {String} page
 * @param {String} elementSelector
 */
function expectNotVisible(page, elementSelector) {
  const expectElement = new Base.ExpectElement(page, elementSelector);
  waitElementPresent(page, elementSelector);
  return expectElement.expectElementNotVisible();
}

/**
 * Expect if element is not found
 * @param {String} page
 * @param {String} elementSelector
 */
function expectNotFound(page, elementSelector) {
  const expectElement = new Base.ExpectElement(page, elementSelector);
  return expectElement.expectElementNotPresent();
}

/**
 * Expect if element is enabled
 * @param {String} page
 * @param {String} elementSelector
 */
function expectEnabled(page, elementSelector) {
  const expectElement = new Base.ExpectElement(page, elementSelector);
  waitElementVisible(page, elementSelector);
  return expectElement.expectElementEnabled();
}

/**
 * Expect if element is not enabled
 * @param {String} page
 * @param {String} elementSelector
 */
function expectNotEnabled(page, elementSelector) {
  const expectElement = new Base.ExpectElement(page, elementSelector);
  waitElementVisible(page, elementSelector);
  return expectElement.expectElementNotEnabled();
}

/**
 * Expect if element is present
 * @param {String} page
 * @param {String} elementSelector
 */
function expectPresent(page, elementSelector) {
  const expectElement = new Base.ExpectElement(page, elementSelector);
  return expectElement.expectElementPresent();
}

/**
 * Expect if checkbox item is selected
 * @param {String} page
 * @param {String} elementSelector
 */
function expectCheckboxSelected(page, elementSelector) {
  const checkbox = new Base.CheckBox(page, elementSelector);
  checkbox.expectCheckboxSelected();
}

/**
 * Expect if radio button item is selected
 * @param {String} page
 * @param {String} elementSelector
 */
function expectRadioButtonSelected(page, elementSelector) {
  const radioButton = new Base.RadioButton(page, elementSelector);
  radioButton.expectRadioButtonSelected();
}
/**
 * Expect if radio button value item is selected
 * @param {String} page
 * @param {String} elementSelector
 */
function expectRadioButtonSelectedValue(page, elementSelector) {
  const radioButton = new Base.RadioButton(page, elementSelector);
  radioButton.expectRadioButtonSelectedValue();
}

/**
 * Check if given label equals the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function expectEqualsTextFromElement(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.expectEqualText();
}

/**
 * Check if given value from form equals the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function expectEqualsValueFromElement(page, elementSelector, expectedValue) {
  const value = '';
  const textField = new Base.TextField(page, elementSelector, value, expectedValue);
  waitElementVisible(page, elementSelector);
  return textField.expectEqualsValue();
}

/**
 * Check if given label Not equals the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function expectNotEqualsTextFromElement(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.expectNotEqualText();
}

/**
 * Check if given element equals the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function expectContainsTextFromElement(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.expectContainsText();
}

/**
 * Check if given label equals the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function expectLabelContainsTextFromElement(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.expectLabelContainsText();
}

/**
 * Check if given paragraph equals the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function expectParagraphContainsTextFromElement(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.expectParagraphContainsText();
}

/**
 * Check if given Span equals the specific text
 * @param {String} page
 * @param {String} elementSelector
 * @param {String} expectedText
 */
function expectSpanContainsTextFromElement(page, elementSelector, expectedText) {
  const label = new Base.Label(page, elementSelector, expectedText);
  waitElementVisible(page, elementSelector);
  return label.expectSpanContainsText();
}

module.exports = {
  setURL,
  switchTabWindow,
  maxWindow,
  scrollToElement,
  clickButton,
  clickButtonOptional,
  clickButtonWithoutWait,
  clickLabel,
  clickLabelOptional,
  clickLinkText,
  clickLinkByText,
  clickLinkTextWithoutWait,
  clickDivElement,
  clickSpanElement,
  clickIconElement,
  clickListElement,
  clickImageElement,
  waitElementVisible,
  waitElementPresent,
  waitUntilElementEnabled,
  waitUntilElementDisabled,
  setValueTextField,
  setValueTextFieldThenEnter,
  setValueTextArea,
  chooseOptionValue,
  chooseOptionClass,
  selectRadioButton,
  selectRadioButtonByValue,
  selectCheckbox,
  selectCheckboxByValue,
  uploadFile,
  keyboardPress,
  keyboardPressWithDelay,
  keyboardPressOnOthers,
  keyboardPressWithDelayOnOthers,
  assertURLEquals,
  assertURLContains,
  assertPageTitle,
  assertContainsText,
  assertLabelContainsText,
  assertParagraphContainsText,
  assertSpanContainsText,
  assertVisible,
  assertListOfElementsContainsText,
  searchTextByElements,
  expectVisible,
  expectNotVisible,
  expectNotFound,
  expectEnabled,
  expectNotEnabled,
  expectPresent,
  expectCheckboxSelected,
  expectRadioButtonSelected,
  expectRadioButtonSelectedValue,
  pauseSleep,
  expectEqualsTextFromElement,
  expectEqualsValueFromElement,
  expectNotEqualsTextFromElement,
  expectContainsTextFromElement,
  expectLabelContainsTextFromElement,
  expectParagraphContainsTextFromElement,
  expectSpanContainsTextFromElement,
};
